'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _gulpUtil = require('gulp-util');

var _gulpUtil2 = _interopRequireDefault(_gulpUtil);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _mergeStream = require('merge-stream');

var _mergeStream2 = _interopRequireDefault(_mergeStream);

var _runSequence = require('run-sequence');

var _runSequence2 = _interopRequireDefault(_runSequence);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

var _tasksTimer = require('../tasks/timer');

var _tasksTimer2 = _interopRequireDefault(_tasksTimer);

var _tasksClean = require('../tasks/clean');

var _tasksClean2 = _interopRequireDefault(_tasksClean);

var gulp = require('gulp');

_runSequence2['default'].use(gulp);
var taskListDefault = {
  clean: _tasksClean2['default'],
  copy: _gulpUtil2['default'].noop //a default through
};
var IsCustomFile = /\./;

var taskList = _lodash2['default'].clone(taskListDefault);

_config2['default'].getGulp = function getGulp() {
  return gulp;
};

_config2['default'].use = function useGulp(gulpContext) {
  gulp = gulpContext || gulp;
  _runSequence2['default'].use(gulp);
  return _config2['default'];
};

_config2['default'].load = function load(tasks) {
  if (_lodash2['default'].isString(tasks)) {
    loadTask(loadPlugin(tasks));
  } else if (_lodash2['default'].isObject(tasks)) {
    loadPluginObject(tasks);
  }

  for (var _len = arguments.length, others = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    others[_key - 1] = arguments[_key];
  }

  if (others && others.length) {
    others.filter(function (task) {
      return _lodash2['default'].isString(task);
    }).map(function (task) {
      return loadTask(loadPlugin(task));
    });
  }

  return _config2['default'];
};

_config2['default'].pipelines = function pipelines(pipelines) {
  Object.keys(pipelines).map(function (taskName) {
    return loadPipeline(taskName, pipelines[taskName]);
  });
  return _config2['default'];
};

_config2['default'].queue = function queue(obj) {
  Object.keys(obj).map(function (key) {
    gulp.task(key, function (cb) {
      _runSequence2['default'].apply(gulp, obj[key].map(function (plugin) {
        return [key, plugin].join('.');
      }).concat(cb));
    });
    return obj[key].map(function (plugin) {
      return loadTask(loadPlugin(plugin, [key, plugin].join('.')));
    });
  });
  return _config2['default'];
};
_config2['default'].reset = function reset() {
  taskList = _lodash2['default'].clone(taskListDefault);
  this['default']();
  gulp.reset();
  return _config2['default'];
};

function loadTask(obj) {
  if (gulp.hasTask(obj.taskName)) return;

  // clean task is special as here
  if (obj.modules[obj.taskName] === _tasksClean2['default']) {
    return gulp.task(obj.taskName, function (cb) {
      (0, _tasksClean2['default'])(_config2['default'].getConf(obj.taskName, 'src') || _config2['default'].getConf('dest'))(cb);
    });
  }

  return gulp.task(obj.taskName, function () {
    var conf = _config2['default'].getConf(obj.taskName);

    // enable multiple entries
    conf = _lodash2['default'].isArray(conf) ? conf : [conf];

    return _mergeStream2['default'].apply(gulp, conf.map(function (config) {
      var srcOption = [].concat(config && config.src || _config2['default'].getConf('src'));

      // src options
      if (config && config.srcOption) {
        srcOption = srcOption.concat(config.srcOption);
      }

      var src = gulp.src.apply(gulp, srcOption);

      // enable pipelines
      Object.keys(obj.modules).forEach(function (key) {
        src = src.pipe(obj.modules[key](config && config[key] || config));
      });

      src = src.on('error', _gulpUtil2['default'].log);

      if (config && config.dest) {
        src = src.pipe(gulp.dest(config.dest || _config2['default'].getConf('dest'), config.destOption));
      }

      if (gulp.isWatching) {
        src = src.pipe((0, _tasksTimer2['default'])(obj.taskName));
      }

      return src;
    }));
  });
}

function loadPipeline(taskName, pipeline) {
  if (gulp.hasTask(taskName)) return;

  var obj = {
    taskName: taskName,
    modules: {}
  };

  // wrap the pipe line into stream
  if (pipeline && pipeline.length) {
    pipeline.forEach(function (item) {
      _lodash2['default'].merge(obj.modules, loadPlugin(item).modules);
    });
  }
  return loadTask(obj);
}

function loadPluginObject(obj) {
  return Object.keys(obj).map(function (key) {
    return loadTask(loadPlugin(obj[key], key));
  });
}

function loadPlugin(pluginName, taskName) {
  return IsCustomFile.test(pluginName) ? loadFromFile(pluginName, taskName) : loadModule(pluginName, taskName);
}

function loadModule(moduleName, taskName) {
  taskName = taskName || moduleName;

  // cache the module or load the default tasks
  taskList[taskName] = taskList[moduleName || taskName] || require(moduleName);

  var obj = {
    taskName: taskName,
    modules: {}
  };
  obj.modules[taskName] = taskList[taskName];

  return obj;
}

function loadFromFile(fileName, taskName) {
  var relative = _path2['default'].relative(__dirname, _path2['default'].resolve(fileName));

  return loadModule(relative, taskName || _path2['default'].basename(fileName));
}

exports['default'] = _config2['default'];
module.exports = exports['default'];